//		----------------------------------------------------------
//		-- fredrik olofsson 04301								--
//		-- ported to pd 060510									--
//		-- converted from abstraction with the same name		--
//		-- distributed under GNU GPL license					--
//		----------------------------------------------------------

#include "m_pd.h"
#include <math.h>

static t_class *this_class;

typedef struct _f0_auto_scale {
	t_object x_ob;
	t_float x_valLeft;
	t_float x_valMiddle;
	t_float x_valRight;
	t_float x_min;
	t_float x_max;
	t_int x_flag;
	t_outlet *x_out;
	t_outlet *x_out2;
} x_f0_auto_scale;

void f0_auto_scale_func(x_f0_auto_scale *f0_auto_scale);

//----------------------------------------------------------------------------------------------
void *f0_auto_scale_new(t_symbol *s, int argc, t_atom *argv) {
	x_f0_auto_scale *f0_auto_scale= (x_f0_auto_scale *)pd_new(this_class);
	floatinlet_new(&f0_auto_scale->x_ob, &f0_auto_scale->x_valMiddle);
	floatinlet_new(&f0_auto_scale->x_ob, &f0_auto_scale->x_valRight);
	f0_auto_scale->x_valLeft= 0;
	f0_auto_scale->x_valMiddle= 0;
	f0_auto_scale->x_valRight= 1;
	f0_auto_scale->x_min= 0;
	f0_auto_scale->x_max= 0;
	f0_auto_scale->x_flag= 0;
	switch(argc) {
		case 2:
			f0_auto_scale->x_valRight= atom_getfloat(argv+1);
		case 1:
			f0_auto_scale->x_valMiddle= atom_getfloat(argv);
	}
	f0_auto_scale->x_out= outlet_new(&f0_auto_scale->x_ob, gensym("float"));
	f0_auto_scale->x_out2= outlet_new(&f0_auto_scale->x_ob, gensym("list"));
	return (void *)f0_auto_scale;
}

//----------------------------------------------------------------------------------------------
void f0_auto_scale_float(x_f0_auto_scale *f0_auto_scale, t_floatarg value) {
	f0_auto_scale->x_valLeft= value;
	f0_auto_scale_func(f0_auto_scale);
}
void f0_auto_scale_bang(x_f0_auto_scale *f0_auto_scale) {
	f0_auto_scale->x_min= 0;
	f0_auto_scale->x_max= 0;
	f0_auto_scale->x_flag= 0;
}
void f0_auto_scale_factor(x_f0_auto_scale *f0_auto_scale) {
	float rangeIn, rangeOut;
	t_atom daList[3];
	rangeIn= fabs(f0_auto_scale->x_max-f0_auto_scale->x_min);
	rangeOut= fabs(f0_auto_scale->x_valRight-f0_auto_scale->x_valMiddle);
	SETFLOAT(daList, 0);				//index for routing
	if(rangeIn==0) {
		SETFLOAT(daList+1, 0);
		SETFLOAT(daList+2, rangeOut);
	} else if(rangeOut==0) {
		SETFLOAT(daList+1, rangeIn);
		SETFLOAT(daList+2, 0);
	} else if(rangeIn<=rangeOut) {
		SETFLOAT(daList+1, 1);
		SETFLOAT(daList+2, 1/(rangeIn/rangeOut));
	} else {
		SETFLOAT(daList+1, 1/(rangeOut/rangeIn));
		SETFLOAT(daList+2, 1);
	}
	outlet_list(f0_auto_scale->x_out2, gensym("list"), 3, daList);
}
void f0_auto_scale_range(x_f0_auto_scale *f0_auto_scale) {
	t_atom daList[3];
	SETFLOAT(daList, 1);				//index for routing
	if(f0_auto_scale->x_min<=f0_auto_scale->x_max) {
		SETFLOAT(daList+1, f0_auto_scale->x_min);
		SETFLOAT(daList+2, f0_auto_scale->x_max);
	} else {
		SETFLOAT(daList+1, f0_auto_scale->x_max);
		SETFLOAT(daList+2, f0_auto_scale->x_min);
	}
	outlet_list(f0_auto_scale->x_out2, gensym("list"), 3, daList);
}
void f0_auto_scale_set(x_f0_auto_scale *f0_auto_scale, t_floatarg min, t_floatarg max) {
	f0_auto_scale->x_min= min;
	f0_auto_scale->x_max= max;
}

//----------------------------------------------------------------------------------------------
void f0_auto_scale_func(x_f0_auto_scale *f0_auto_scale) {
	float v, o, rangeIn, rangeOut;
	v= f0_auto_scale->x_valLeft;
	if(f0_auto_scale->x_flag==0&&f0_auto_scale->x_min==f0_auto_scale->x_max) {
		f0_auto_scale->x_flag= 1;
		f0_auto_scale->x_min= v;
		f0_auto_scale->x_max= v;
	}
	if(v<f0_auto_scale->x_min) {
		f0_auto_scale->x_min= v;
	}
	if(v>f0_auto_scale->x_max) {
		f0_auto_scale->x_max= v;
	}
	rangeIn= fabs(f0_auto_scale->x_max-f0_auto_scale->x_min);
	rangeOut= fabs(f0_auto_scale->x_valRight-f0_auto_scale->x_valMiddle);
	if(rangeIn==0) {
		if(f0_auto_scale->x_valMiddle<=f0_auto_scale->x_valRight) {
			o= f0_auto_scale->x_valMiddle;
		} else {
			o= f0_auto_scale->x_valRight;
		}
	} else if(f0_auto_scale->x_valMiddle<=f0_auto_scale->x_valRight) {
		o= fabs((v-f0_auto_scale->x_min)/rangeIn*rangeOut)+f0_auto_scale->x_valMiddle;
	} else {
		o= fabs((v-f0_auto_scale->x_max)/rangeIn*rangeOut)+f0_auto_scale->x_valRight;
	}
	outlet_float(f0_auto_scale->x_out, o);
}

//----------------------------------------------------------------------------------------------
void f0_auto_scale_setup(void) {
	this_class= class_new(gensym("f0_auto_scale"), (t_newmethod)f0_auto_scale_new, 0, sizeof(x_f0_auto_scale), 0, A_GIMME, 0);
	class_addbang(this_class, (t_method)f0_auto_scale_bang);
	class_addfloat(this_class, f0_auto_scale_float);
	class_addmethod(this_class, (t_method)f0_auto_scale_factor, gensym("factor"), 0);
	class_addmethod(this_class, (t_method)f0_auto_scale_range, gensym("range"), 0);
	class_addmethod(this_class, (t_method)f0_auto_scale_set, gensym("set"), A_DEFFLOAT, A_DEFFLOAT, 0);
	post("f0_auto_scale v1.1  distributed under GNU GPL license");
}
